//
//  BGLocation.m
//  BGLocation
//
//  Created by Eric on 2017/8/22.
//  Copyright © 2017年 eric. All rights reserved.
//


#import "RCTBridge.h"
#import "RCTEventDispatcher.h"
#import "BGLocation.h"
#import "BGTask.h"
@interface BGLocation()
{
    BOOL isCollect;
}
@property (strong , nonatomic) BGTask *bgTask; //后台任务
@property (strong , nonatomic) NSTimer *restarTimer; //重新开启后台任务定时器
@property (strong , nonatomic) NSTimer *closeCollectLocationTimer; //关闭定位定时器 （减少耗电）
@property (nonatomic) double customDistanceFilter; // 客制化距离过滤参数
@property (strong, nonatomic) CLLocation* currentLocation; //当前地理位置

@end
@implementation BGLocation
//@synthesize bridge = _bridge;
RCT_EXPORT_MODULE();
//初始化
-(instancetype)init
{
    if(self == [super init])
    {
        //
        _bgTask = [BGTask shareBGTask];
        isCollect = NO;
        _customDistanceFilter = 50;
        //监听进入后台通知
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationEnterBackground) name:UIApplicationDidEnterBackgroundNotification object:nil];
        //监听前台
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationBecomeActive) name:UIApplicationDidBecomeActiveNotification object:nil];
    }
    return self;
}

-(instancetype)initWithCustomDistanceFilter:(double)customDistanceFilter
{
    if(self == [super init])
    {
        //
        _bgTask = [BGTask shareBGTask];
        isCollect = NO;
        _customDistanceFilter = customDistanceFilter;
        //监听进入后台通知
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationEnterBackground) name:UIApplicationDidEnterBackgroundNotification object:nil];
        //监听前台
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationBecomeActive) name:UIApplicationDidBecomeActiveNotification object:nil];
    }
    return self;
}

- (NSArray<NSString *> *)supportedEvents {
    return @[@"kCLErrorDenied", @"didUpdateLocations"];
}

+(CLLocationManager *)shareBGLocation
{
    static CLLocationManager *_locationManager;
    static dispatch_once_t predicate;
    dispatch_once(&predicate, ^{
        _locationManager = [[CLLocationManager alloc] init];
        _locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        _locationManager.allowsBackgroundLocationUpdates = YES;
        _locationManager.pausesLocationUpdatesAutomatically = NO;
    });
    return _locationManager;
}

//监听前台
-(void)applicationBecomeActive
{
    NSLog(@"come in foreground");
    CLLocationManager *locationManager = [BGLocation shareBGLocation];
    locationManager.delegate = self;
    locationManager.distanceFilter = kCLDistanceFilterNone; // 不移动也可以后台刷新回调
    if ([[UIDevice currentDevice].systemVersion floatValue]>= 8.0) {
        [locationManager requestAlwaysAuthorization];
    }
    [locationManager startUpdatingLocation];
}


//后台监听方法
-(void)applicationEnterBackground
{
    NSLog(@"come in background");
    CLLocationManager *locationManager = [BGLocation shareBGLocation];
    locationManager.delegate = self;
    locationManager.distanceFilter = kCLDistanceFilterNone; // 不移动也可以后台刷新回调
    if ([[UIDevice currentDevice].systemVersion floatValue]>= 8.0) {
        [locationManager requestAlwaysAuthorization];
    }
    [locationManager startUpdatingLocation];
    [_bgTask beginNewBackgroundTask];
}

//开启服务
- (void)startLocation {
    NSLog(@"开启定位");
    
    if ([CLLocationManager locationServicesEnabled] == NO) {
        NSLog(@"locationServicesEnabled false");
        UIAlertView *servicesDisabledAlert = [[UIAlertView alloc] initWithTitle:@"Location Services Disabled" message:@"You currently have all location services for this device disabled" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [servicesDisabledAlert show];
    } else {
        CLAuthorizationStatus authorizationStatus= [CLLocationManager authorizationStatus];
        
        if(authorizationStatus == kCLAuthorizationStatusDenied || authorizationStatus == kCLAuthorizationStatusRestricted){
            NSLog(@"authorizationStatus failed");
        } else {
            NSLog(@"authorizationStatus authorized");
            CLLocationManager *locationManager = [BGLocation shareBGLocation];
            locationManager.distanceFilter = kCLDistanceFilterNone;
            
            if([[UIDevice currentDevice].systemVersion floatValue]>= 8.0) {
                [locationManager requestAlwaysAuthorization];
            }
            [locationManager startUpdatingLocation];
        }
    }
}


//重启定位服务
-(void)restartLocation
{
    NSLog(@"重新启动定位");
    CLLocationManager *locationManager = [BGLocation shareBGLocation];
    locationManager.delegate = self;
    locationManager.distanceFilter = kCLDistanceFilterNone; // 不移动也可以后台刷新回调
    if ([[UIDevice currentDevice].systemVersion floatValue]>= 8.0) {
        [locationManager requestAlwaysAuthorization];
    }
    [locationManager startUpdatingLocation];
    [self.bgTask beginNewBackgroundTask];
}


//停止后台定位
-(void)stopLocation
{
    NSLog(@"停止定位");
    isCollect = NO;
    CLLocationManager *locationManager = [BGLocation shareBGLocation];
    [locationManager stopUpdatingLocation];
}
#pragma mark --delegate
//定位回调里执行重启定位和关闭定位
-(void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray<CLLocation *> *)locations
{
    NSLog(@"定位收集");
    CLLocation* lastLocation = [locations lastObject];
    if (_currentLocation != nil) {
        CLLocationDistance distance = [_currentLocation distanceFromLocation:lastLocation];
        NSDate* currentTimeStamp = _currentLocation.timestamp;
        NSDate* lastTimeStamp = lastLocation.timestamp;
        NSTimeInterval interval = [lastTimeStamp timeIntervalSinceDate:currentTimeStamp];
        if (distance > _customDistanceFilter) {
             _currentLocation = lastLocation;
            NSLog(@"移动距离超过阈值发送经纬度给react native");
            [self sendEventWithName:@"didUpdateLocations"
                                                         body:@{@"latitude": [NSNumber numberWithDouble: lastLocation.coordinate.latitude], @"longitude": [NSNumber numberWithDouble:lastLocation.coordinate.longitude]}];
        } else if(interval > 600) {
            _currentLocation = lastLocation;
            NSLog(@"静止时间超过阈值10分钟强制发送经纬度给react native");
            [self sendEventWithName:@"didUpdateLocations"
                               body:@{@"latitude": [NSNumber numberWithDouble: lastLocation.coordinate.latitude], @"longitude": [NSNumber numberWithDouble:lastLocation.coordinate.longitude]}];
        }
    } else {
        _currentLocation = lastLocation;
        NSLog(@"初始经纬度发送给react native");
        [self sendEventWithName:@"didUpdateLocations"
                                                         body:@{@"latitude": [NSNumber numberWithDouble: lastLocation.coordinate.latitude], @"longitude": [NSNumber numberWithDouble:lastLocation.coordinate.longitude]}];
    }
    
    
    //[self sendLocation:[locations objectAtIndex:0]];
    
    //如果正在10秒定时收集的时间，不需要执行延时开启和关闭定位
    if (isCollect) {
        return;
    }
    [self performSelector:@selector(restartLocation) withObject:nil afterDelay:120];
    [self performSelector:@selector(stopLocation) withObject:nil afterDelay:10];
    isCollect = YES;//标记正在定位
}
- (void)locationManager: (CLLocationManager *)manager didFailWithError: (NSError *)error
{
    // NSLog(@"locationManager error:%@",error);
    
    switch([error code])
    {
        case kCLErrorNetwork: // general, network-related error
        {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"网络错误" message:@"请检查网络连接" delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil, nil];
            [alert show];
        }
            break;
        case kCLErrorDenied:{
            //      UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"请开启后台服务" message:@"应用不可以定位，需要在在设置/通用/后台应用开启定位" delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil, nil];
            //      [alert show];
            
            
            //未开启定位权限，向JS端发送event，由JS端负责处理
            NSLog(@"未开启定位权限，向JS端发送event，由JS端负责处理");
            [self sendEventWithName:@"kCLErrorDenied"
                                                         body:@{@"title": @"请开启定位服务", @"message": @"应用不可以定位，需要在在设置/通用/后台应用开启定位"}];
        }
            break;
        default:
        {
            
        }
            break;
    }
}



-(void)startLocationWatch
{
    BGTask* bgTask = [BGTask shareBGTask];
    UIAlertView *alert;
    //判断定位权限
    if([UIApplication sharedApplication].backgroundRefreshStatus == UIBackgroundRefreshStatusDenied)
    {
        alert = [[UIAlertView alloc]initWithTitle:@"提示" message:@"应用没有不可以定位，需要在在设置/通用/后台应用刷新开启" delegate:nil cancelButtonTitle:@"确定" otherButtonTitles:nil, nil];
        [alert show];
    }
    else if ([UIApplication sharedApplication].backgroundRefreshStatus == UIBackgroundRefreshStatusRestricted)
    {
        alert = [[UIAlertView alloc]initWithTitle:@"提示" message:@"设备不可以定位" delegate:nil cancelButtonTitle:@"确定" otherButtonTitles:nil, nil];
        [alert show];
    }
    else
    {
        //BGLocation* bgLocation = [[BGLocation alloc]init];
        [self startLocation];
        CLLocationManager* clmanager = [BGLocation shareBGLocation];
        [clmanager startMonitoringSignificantLocationChanges];
        
        [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(log) userInfo:nil repeats:YES];
    }
}

-(void)log
{
    NSLog(@"Program running...");
}


RCT_EXPORT_METHOD(bgLocationWatchStart)
{
    NSLog(@"BgLocationWatchStart");

    dispatch_sync(dispatch_get_main_queue(), ^{
        [self startLocationWatch];
    }); 
}



@end
